/******************************************************
 *  전자공작 제작 예	시계
 *　PIC16LF1709	RTC8564 사용
 *　온도 센서　　S8100B
 ******************************************************/
#include <xc.h>
#include "lcd_lib2.h"
#include "i2c_lib2.h"
/* 컨피규레이션의 정의 */
// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset enabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover Mode (Internal/External Switchover Mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PPS1WAY = OFF    // Peripheral Pin Select one-way control (The PPSLOCK bit can be set and cleared repeatedly by software)
#pragma config ZCDDIS = ON      // Zero-cross detect disable (Zero-cross detect circuit is disabled at POR)
#pragma config PLLEN = OFF      // Phase Lock Loop enable (4x PLL is enabled when software sets the SPLLEN bit)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)
/** 전역 변수 정수 정의 ***/
unsigned char StMsg[] = "**Start Clock** ";
unsigned char Mesg[] = " mm/dd hh:mm:ss ";
unsigned char TMsg[] = " Ondo xx.x 'C   ";
unsigned char SetMsg[] = "Setting  Y = xx ";

unsigned char Buffer[8];
unsigned char Year, Month, Day, Hour, Min, Sec, Week;
unsigned char Select, SW, Cflag, Flag;
int temp;
float Ondo;

/* 함수 프로토타입 */
unsigned char CheckKey(void);
int Mesure(unsigned char ch);
void CurDisp(void);
void Settings(void);
unsigned char BCD(unsigned char data);
void itostring(unsigned char digit, unsigned int data, unsigned char *buffer);
void ftostring(int seisu, int shousu, float data, unsigned char *buffer);

/************** 메인 함수  *************************/
int main(void) {
    /* 클록 설정 */
    OSCCONbits.IRCF = 13;               // 4MHz
    /* I/O 초기 설정 */
    LATA = 0;                           // LED Off
    LATC = 0;                           // LED Off
    ANSELA = 0;                         // 모두 디지털
    ANSELB = 0;
    ANSELC = 0xC8;                      // RC3,6,7 아날로그
    TRISA = 0x2F;
    TRISB = 0x50;
    TRISC = 0xCF;                       // 스위치 입력
    WPUC = 0x07;                        // 스위치 RC 0, 1, 2 풀업
    WPUAbits.WPUA2 = 1;                 // 스위치 RA2만 풀업
    OPTION_REGbits.nWPUEN = 0;          // 풀업 유효화
    /* 오피 앰프 2 초기 설정 */
    OPA2CON = 0x90;                     // Unity Gain
    FVRCON = 0x82;                      // FVR1 = 2.048V FVR2 = Off
    /* A/D 컨버터 초기 설정 */
    ADCON1 = 0x93;                      // 오른쪽 정렬, Fosc/8, Vref=FVR1
    ADCON2 = 0;                         // 매뉴얼 변환
    /** I2C 초기화 **/
    RB4PPS = 0x11;                      // SDA1핀 할당
    RB6PPS = 0x10;                      // SCL1핀 할당
    SSP1ADD = 0x4F;                     // 100kHz 이하
    SSP1CON1 = 0x38;                    // I2C Enable
    SSP1STAT = 0;                       // 상태 클리어
    /* 액정 표시기 초기화 */
    LATBbits.LATB5 = 1;                 // LCD 전원 ON
    lcd_init();                         // LCD 초기화
    lcd_clear();                        // 전체 소거
    lcd_str(StMsg);                     // 시작 메시지 표시
    /* RTC 초기 설정 */
    CmdI2C(0xA2, 0x00, 0x20);           // Stop
    CmdI2C(0xA2, 0x0D, 0x83);           // 초 펄스 출력 설정
    CmdI2C(0xA2, 0x00, 0x00);           //Start
    /* 상태 변화 끼어들기 설정(RA   초 끼어들기)  */
    IOCAPbits.IOCAP5 = 1;               // 상승 에지 유효화
    IOCAFbits.IOCAF5 = 0;               // 플래그 클리어
    INTCONbits.IOCIF = 0;               // 플래그 클리어
    INTCONbits.IOCIE = 1;               // 상태 변화 끼어들기 허용
    INTCONbits.PEIE = 1;                // 주변 끼어들기 허용
    INTCONbits.GIE = 1;                 // 끼어들기 허용
    /** 변수 초기화　**/
    Cflag = 0;                          // 시각 설정 중 플래그 리셋
    
    /************ 메인 루프　　*****************/
    while(1)
    {
        /** 시각 표시 처리 ***/
        if(Flag == 1){                          // 끼어들기가 있을 때
            Flag = 0;                           // 초 플래그 클리어
            SendI2C(0xA2, 0x02);                // 레지스터 포인터 설정
            GetDataI2C(0xA2, Buffer, 7);        // 7byte Read　년부터 초까지
            Year = ((Buffer[6]&0xF0) >> 4)*10;  // 년을 BCD에서 바이너리로
            Year += Buffer[6] & 0x0F;
            Month = ((Buffer[5]&0x10) >> 4)*10; // 월을 BCD에서 바이너리로
            Month += Buffer[5] & 0x0F;
            itostring(2, Month, Mesg+1);        // 월을 문자로 변환
            Day = ((Buffer[3]&0x30)>>4)*10;     // 일을 BCD에서 바이너리로
            Day += Buffer[3]&0x0F;
            itostring(2, Day, Mesg+4);          // 일을 문자로 변환
            Hour = ((Buffer[2]&0x30) >> 4)*10;  // 시간을 BCD에서 바이너리로
            Hour+= Buffer[2] & 0x0F;
            itostring(2, Hour, Mesg+7);         // 시간을 문자로 변환
            Min = ((Buffer[1]&0x70) >> 4)*10;   // 분을 BCD에서 바이너리로
            Min += Buffer[1] & 0x0F;
            itostring(2, Min, Mesg+10);         // 분을 문자로 변환
            Sec = ((Buffer[0]&0x70) >> 4) * 10; // 초를 BCD에서 바이너리로
            Sec += Buffer[0] & 0x0F;
            itostring(2, Sec, Mesg+13);         // 초를 문자로 변환
            Mesg[15] = ' ';                     // 스페이스
            lcd_cmd(0x80);                      // 1행 지정
            lcd_str(Mesg);                      // 월일시분초 표시
            /**** 온도 계측 표시	10초마다 *****/
            if((Sec%10) == 0){                      // 10초마다
                temp = Mesure(7);                   // A/D 계측
                Ondo = (2.048 * temp) / 1024;       // 전압으로 변환
                Ondo = -123.0 * Ondo + 214.7;       // 온도로 변환
                ftostring(2, 1, Ondo, TMsg+6);      // 문자로 변환
                TMsg[11] = 0xDF;                    // ° 문자
                lcd_cmd(0xC0);                      // 2행 지정
                lcd_str(TMsg);                      // 온도 표시
            }
           /******** 스위치 입력 검사 ********/
            SW = CheckKey();                        // 스위치 입력
            if(SW == 1){                            // SW1인가?
                Cflag = 1;                          // 갱신 중 flag on
                Settings();                         // 시각 설정 처리로
            }
        }
        /******* SLEEP 모드 진입 *********/
        SLEEP();
    }
}
/*************************************
 *　상태 변화 끼어들기 처리 함수
 ************************************/
void interrupt IOCPROC(void){
    Flag = 1;                       // 플래그 설정
//    LATAbits.LATA4 ^= 1;          // 표시 점등
    INTCONbits.IOCIF = 0;           // 끼어들기 플래그 클리어
    IOCAFbits.IOCAF5 = 0;           // 끼어들기 플래그 클리어
}
/********************************************
 * A/D 변환 서브 함수
 *******************************************/
int Mesure(unsigned char ch){
    int value;
    
    ADCON0 = (ch <<2) | 0x01;       // ADC On Select Channel
    Delayus(100);                   // 안정화 대기
    ADCON0bits.GO = 1;              // Start
    while(ADCON0bits.GO);           // acquisition time
    value = ADRESH*256+ADRESL;      // 10비트 바이너리로 변환
    ADCON0 = 0;                     // ADC Off
    return(value);                  // 값을 반환
}
/****************************************
*  스위치 입력 서브 함수
*  반환값에 스위치 번호
***************************************/
unsigned char CheckKey(void){
    unsigned char Key;

    Key = 0;                        // 초깃값 클리어
    if(PORTCbits.RC2 == 0)          // Select 스위치
        Key = 1;                    // 1을 반환
    if(PORTCbits.RC1 == 0)          // Up 스위치
        Key = 2;                    // 2를 반환
    if(PORTCbits.RC0 == 0)          // Down 스위치
        Key = 3;                    // 3을 반환
    if(PORTAbits.RA2 == 0)          // Enter 스위치
        Key = 4;                    // 4를 반환
    return(Key);
}
/********************************************
*  현재 값 시각 표시 서브 함수(설정 중 표시)
********************************************/
void CurDisp(void){
    SetMsg[15] = ' ';
    switch(Select){
        case 0:                             // 년일 때
            SetMsg[9] = 'Y';
            itostring(2, Year, SetMsg+13);  // 현재 값을 문자로 변환
            break;
        case 1:                             // 월일 때
            SetMsg[9] = 'M';
            itostring(2, Month, SetMsg+13); // 현재 값을 문자로 변환
            break;
        case 2:                             // 일일 때
            SetMsg[9] = 'D';
            itostring(2, Day, SetMsg+13);   // 현재 값을 문자로 변환
            break;;
        case 3:                             // 시일 때
            SetMsg[9] = 'H';
            itostring(2, Hour, SetMsg+13);  // 현재 값을 문자로 변환
            break;
        case 4:                             // 분일 때
            SetMsg[9] = 'm';
            itostring(2, Min, SetMsg+13);   // 현재 값을 문자로 변환
            break;
        case 5:                             // 초일 때
            SetMsg[9] = 'S';
            itostring(2, Sec, SetMsg+13);   // 현재 값을 문자로 변환
            break;
        default:
            break;
    }
    lcd_cmd(0x80);                          // 1행 지정
    lcd_str(SetMsg);                        // 설정값 표시
}
/*******************************************
*  시각 설정 처리
*　Key：Select, Up, Down, Set
*******************************************/
void Settings(void){
    Select = 0;                             // 년으로 초기 설정
    CurDisp();                              // 현재 값 표시
    while(CheckKey() != 0);                 // 채터링 방지
    Delayms(25);
    /*** 시각 설정 중 루프  ****/
    while(Cflag){                           // 갱신 중 플래그 ON일 때의 루프
        SW = CheckKey();                    // 스위치 입력
        while(CheckKey() != 0);             // 채터링 방지
        Delayms(25);
        /* 스위치별 프로세스 */
        switch(SW){                         // SW 값으로 분기
            case 1:
                Select++;                   // 선택 항목 갱신
                if(Select > 5)              // 상한 체크
                    Select = 0;             // 맨 처음으로 되돌림
                CurDisp();                  // 현재 값 표시
                break;
            case 2:
                switch(Select){             // Up 처리
                    case 0:
                        Year++;             // 년 Up
                        if(Year > 99)       // 최댓값 99
                            Year = 0;       // 0으로 돌아감
                        break;
                    case 1:
                        Month++;            // 월 Up
                        if(Month > 12)      // 최댓값 12
                            Month = 1;      // 1로 돌아감
                        break;
                    case 2:                 // 일 Up
                        Day++;
                        if(Day > 31)        // 최댓값 31
                            Day = 1;        // 1로 돌아감
                        break;
                    case 3:
                        Hour++;             // 시간 Up
                        if(Hour > 23)       // 최댓값 23
                            Hour = 0;       // 0으로 돌아감
                        break;
                    case 4:
                        Min++;              // 분 Up
                        if(Min > 59)        // 최댓값 59
                            Min = 0;        // 0으로 돌아감
                        break;
                    case 5:
                        Sec++;              // 초 Up
                        if(Sec > 59)        // 59초가 최대
                            Sec = 0;        // 0으로 돌아감
                        break;
                    default: break;
                }
                CurDisp();                  // 갱신값 표시
                break;
            case 3:
                switch(Select){             // Down 처리
                    case 0:
                        if(Year > 0)        // 년 Down
                            Year--;
                        break;
                    case 1:
                        Month--;            // 월 Down
                        if(Month < 1)
                            Month = 12;
                        break;
                    case 2:
                        Day--;              // 일 Down
                        if(Day < 1)
                            Day = 31;
                        break;
                    case 3:
                        if(Hour > 0)        // 시간 Down
                            Hour--;
                        else
                            Hour = 23;
                        break;
                    case 4:
                        if(Min > 0)         // 분 Down
                            Min--;
                        else
                            Min = 59;
                        break;
                    case 5:
                        if(Sec > 0)         // 초 Down
                            Sec--;
                        else
                            Sec = 59;
                        break;
                    default: break;
                }
                CurDisp();                  // 갱신값 표시
                break;
            case 4:                                 // 설정 완료 처리
                /** 갱신 처리 **/
                lcd_clear();
                CmdI2C(0xA2, 0x00, 0x20);           // 일단 정지
                CmdI2C(0xA2, 0x02, BCD(Sec));       // 초 설정
                CmdI2C(0xA2, 0x03, BCD(Min));       // 분 설정
                CmdI2C(0xA2, 0x04, BCD(Hour));      // 시 설정
                CmdI2C(0xA2, 0x05, BCD(Day));       // 일 설정
                CmdI2C(0xA2, 0x06, 0);              // 요일 설정
                CmdI2C(0xA2, 0x07, BCD(Month));     // 월 설정
                CmdI2C(0xA2, 0x08, BCD(Year));      // 년 설정
                CmdI2C(0xA2, 0x00, 0x00);           // 재시작
                Cflag = 0;                          // 갱신 중 플래그 OFF
                break;
            default: break;
        }
    }
}
/**** 바이너리에서 BCD로 변환 *****/
unsigned char BCD(unsigned char data)
{
    unsigned char flow;
    flow = 0;
    flow = (data / 10) << 4;        // 윗자리 수
    flow += (data % 10);            // 아랫자리 수
    return(flow);
}
/***************************************
* int 정수에서 ASCII 문자로 변환
****************************************/
void itostring(unsigned char digit, unsigned int data, unsigned char *buffer){
    unsigned char i;

    buffer += digit;                            // 가장 마지막 숫자 위치
    for(i=digit; i>0; i--) {                    // 변수는 하위부터 상위로
        buffer--;                               // 포인터 -1
        *buffer = (data % 10) + '0';            // ASCII로
        data = data / 10;                       // 다음 자릿수로
    }
}
/***********************************
* Float에서 문자열로 변환
* 합계 유효 단위는 8 자릿수 이하로 할 것
***********************************/
void ftostring(int seisu, int shousu, float data, unsigned char *buffer)
{
    int i;
    long dumy;

    if(shousu != 0)                         // 소수 부분이 있는가?
        buffer += seisu+shousu+1;           // 전체 자릿수+소수점
    else                                    // 소수 부분이 없을 때
        buffer += seisu + shousu;           // 전체 자릿수만
    buffer--;                               // 배열 포인터 -1
    for(i=0; i<shousu; i++)                 // 소수 자리를 정수로 변환
        data = data * 10;                   // 10배
    /* dumy가 오버플로우하면 변환 불가(8자리가 한계) */
    dumy = (long) (data + 0.5);             // 반올림해서 정수로 변환
    for(i=shousu; i>0; i--)	{               // 소수 자릿수만큼 반복
        *buffer =(dumy % 10)+'0';           // 수치를 문자로 변환 저장
        buffer--;                           // 저장 위치 하위에서 상위로
        dumy /= 10;                         // 다음 자릿수로
    }
    if(shousu != 0) {                       // 소수 부분이 0이면 소수점 없음
        *buffer = '.';                      // 소수점을 저장
        buffer--;                           // 포인터 -1
    }
    for(i=seisu; i>0; i--) {                // 정수 자릿수만큼 반복
        *buffer = (dumy % 10)+'0';          // 수치를 문자로 변환 저장
        buffer--;                           // 포인터 -1
        dumy /= 10;                         // 다음 자릿수로
    }
    i = 0;                                  // 블랭크 처리
    buffer++;
    while((i<seisu-1)&&(*buffer == '0')){   // 상위 자릿수가 0인 동안
        *buffer = ' ';                      // 블랭크로 변환
        buffer++;
        i++;
    }
}

