/******************************************
*  I2C통신 라이브러리
******************************************/
#include <xc.h>
#include "i2c_lib2.h"

/*************************************
* I2C 버스를 사용한 데이터 전송
**************************************/
void SendI2C(unsigned char Adrs, unsigned char Data)
{
    IdleI2C();
    SSP1CON2bits.SEN = 1;   		// Start 출력
    while(SSP1CON2bits.SEN);		// Start 종료 대기
    SendByte(Adrs);                    // 주소 + 전송 모드 전송
    SendByte(Data);                    // 명령어 데이터 전송
    SSP1CON2bits.PEN = 1;   		// Stop 출력
    while(SSP1CON2bits.PEN);		// Stop 종료 대기
}
/****************************************
* I2C버스를 사용한 명령어 전송
*****************************************/
void CmdI2C(unsigned char Adrs, unsigned char Reg, unsigned char Data)
{
    IdleI2C();
    SSP1CON2bits.SEN = 1;   		// Start 출력
    while(SSP1CON2bits.SEN);		// Start 종료 대기
    SendByte(Adrs);                    // 주소 + 전송 모드 전송
    SendByte(Reg);                     // 레지스터 지정 전송
    SendByte(Data);                    // 명령어 데이터 전송
    SSP1CON2bits.PEN = 1;   		// Stop 출력
    while(SSP1CON2bits.PEN);		// Stop 종료 대기
}
/** Idle 대기 서브 함수　***/
void IdleI2C(void){
    while(SSP1CON2bits.SEN || SSP1CON2bits.PEN || SSP1CON2bits.RCEN
            || SSP1CON2bits.ACKEN || SSP1STATbits.R_nW);
}
/****************************************
* 　I2C에서 1 바이트 전송
*****************************************/
void SendByte(unsigned char ch){
    SSP1BUF = ch;			  // 데이터 세트 전송 시작
    while(SSP1STATbits.BF);		    // 송신 종료 대기
    while(SSP1CON2bits.ACKSTAT);    // ACK 회신 대기
    IdleI2C();			      // Idle 대기    
}
/*****************************************
* I2C 버스를 사용한 n 바이트 수신
******************************************/
void GetDataI2C(unsigned char Adrs, unsigned char *Buffer, unsigned char Cnt)
{
    unsigned char i;

    IdleI2C();				// Idle 대기
    SSP1CON2bits.SEN = 1;   		// Start 출력
    while(SSP1CON2bits.SEN);		// Start 종료 대기
    SendByte(Adrs+1);                   // 주소 + 수신 모드 전송    
    for(i=0; i<Cnt; i++)
        Buffer[i] = RcvByte(0);         // 1 바이트 수신 ACK 응답
    Buffer[i] = RcvByte(1);             // 마지막 바이트 수신 NACK 응답
    SSP1CON2bits.PEN = 1;   		// Stop 출력
    while(SSP1CON2bits.PEN);		// Stop 종료 대기
}
/****************************************
* 　I2C에서 1 바이트 수신 ACK / NACK 반송
*****************************************/
unsigned char RcvByte(unsigned char Ans){
    unsigned char data;
    
    SSP1CON2bits.ACKDT = Ans;		// ACK(0)/Nack(1) 설정
    SSP1CON2bits.RCEN = 1;		// 수신 허용 RCEN 세트
    while(!SSP1STATbits.BF);            // 수신 대기 BF 체크
    SSP1CON2bits.ACKEN = 1;		// ACK 회신
    data = SSP1BUF;			// 수신 데이터 취득
    SSP1CON1bits.SSPOV = 0;		// 오류 클리어
    IdleI2C();				// Idle 대기
    return(data);
}
/*********************************************************
* 지연 함수  1msec 1usec   Fcy = 1MHz　타이머 4를 사용
**********************************************************/
void Delayms( unsigned int t){
    T4CON = 0x06;             // enable Timer4, Tcy, 1:16
    PR4 = 62;
    while (t--)    {
        while (PIR2bits.TMR4IF == 0);
        PIR2bits.TMR4IF = 0;
    }
}
void Delayus(unsigned int t)
{
    while(t--);
}