/***********************************************
*  액정 표시기 라이브러리
*  I2C 인터페이스
*    lcd_init()    ----- 초기화
*    lcd_cmd(cmd)  ----- 명령어 출력
*    lcd_data(data) ----- 1 문자 표시 출력
*    lcd_str(ptr)  ----- 문자열 표시 출력
*    lcd_clear()   ----- 전체 삭제
**********************************************/
#include <xc.h>
#include "lcd_lib2.h"
#include "i2c_lib2.h"

/*********************************
* 액정에 1 문자 표시 데이터 출력
*********************************/
void lcd_data(unsigned char data)
{
    CmdI2C(0x7C, 0x40, data);
    Delayus(30);                        // 지연
}

/*******************************
* 액정에 1 명령어 출력
*******************************/
void lcd_cmd(unsigned char cmd)
{
    CmdI2C(0x7C, 0x00, cmd);
    /* Clear 또는 Home */
    if((cmd == 0x01)||(cmd == 0x02))
        Delayms(2);			// 2msec 대기
    else
        Delayus(30);			// 30μsec 대기
}
/*******************************
*  초기화 함수
*******************************/
void lcd_init(void)
{
    Delayms(100);
    lcd_cmd(0x38);			// 8bit 2line Normal mode
    lcd_cmd(0x39);			// 8bit 2line Extend mode
    lcd_cmd(0x14);			// OSC 183Hz BIAS 1/5
    /* 콘트라스트 설정 */
    lcd_cmd(0x70 + (CONTRAST & 0x0F));
    lcd_cmd(0x5C + (CONTRAST >> 4));
    lcd_cmd(0x6B);			// Follower
    Delayms(300);
    lcd_cmd(0x38);			// Set Normal mode
    lcd_cmd(0x0C);			// Display On
    lcd_cmd(0x01);			// Clear Display
}
/******************************
* 전체 삭제 함수
******************************/
void lcd_clear(void)
{
    lcd_cmd(0x01);			// 초기화 명령어 출력
}
/*****************************
* 문자열 표시 함수
*****************************/
void lcd_str(const unsigned char* ptr)
{
    while(*ptr != 0)			// 문자 추출
        lcd_data(*ptr++);		// 문자 표시
}
/*****************************
*  아이콘 표시 제어 함수
*****************************/
void lcd_icon(unsigned char num, unsigned char onoff)
{
    lcd_cmd(0x39);			// Extend mode
    lcd_cmd(0x40 | ICON[num][0]);	// 아이콘 주소 지정
    if(onoff)
        lcd_data(ICON[num][1]);         // 아이콘 온 데이터
    else
        lcd_data(0x00);					// 아이콘 오프 데이터
    lcd_cmd(0x38);			// Normal Mode
}
